import type { BaseQueryFn } from '@reduxjs/toolkit/query';

import { getAuthAttribute } from '@prokyon/auth/slice';
import { AUTH_REDUCER_NAME, AuthRootState } from '@prokyon/auth/types';
import { isJSON } from '@prokyon/utils';

import { buildUrl } from './utils';

export const baseQuery =
	<Response = unknown, RequestBody = any>({
		baseUrl,
		authorization = { headerName: 'Authorization', tokenType: 'Bearer' },
	}: {
		authorization?: { headerName: string; tokenType: string };
		baseUrl: string;
	}): BaseQueryFn<
		{
			body?: RequestBody;
			headers?: Record<string, string | number | boolean>;
			method?: string;
			params?: any;
			url: string;
		},
		Response
	> =>
	async ({ url, method = 'GET', params, body, headers }, { getState }) => {
		if ((getState() as AuthRootState).prokyon[AUTH_REDUCER_NAME] !== undefined) {
			const isAuthenticated = getAuthAttribute(getState() as AuthRootState, 'authenticated');
			if (isAuthenticated) {
				const tokenValue = getAuthAttribute(getState() as AuthRootState, 'accessToken');
				headers = {
					...headers,
					[authorization.headerName]: `${authorization.tokenType} ${tokenValue?.toString()}`.trim(),
				};
			}
		}

		try {
			const endpoint = `${baseUrl}${buildUrl(url, params)}`;
			const response = await fetch(endpoint, {
				headers: {
					'Content-Type': 'application/json',
					...headers,
				},
				method: method.toUpperCase(),
				body: JSON.stringify(body),
			});

			const json = await response.text().then((text: string) => {
				return isJSON(text) ? JSON.parse(text) : text;
			});

			if (!response.ok) {
				throw response;
			}

			return { data: json };
		} catch (err) {
			return {
				error: {
					status: err.response?.status,
					data: err.response?.data || err.message,
				},
			};
		}
	};
