import React, { FocusEvent, useCallback, useEffect, useState } from 'react';
import { omit } from 'ramda';
import { format as formatFn } from 'date-fns';

import { ModalId } from '@prokyon/components/Modal/context';
import { useModal } from '@prokyon/components/Modal/hooks';
import { InputElementType } from '@prokyon/types/form';

import { TextField } from '../../TextField';
import { DatePickerModal } from './DatePickerModal';

export const DefaultDateFormat = 'dd.MM.yyyy';

export type DatePickerProps<V = string> = Omit<
	{
		format?: string;
	} & InputElementType<V>,
	'onFocus'
>;

export const DatePicker: React.FC<DatePickerProps<Date>> = (props) => {
	const { onBlur, onChange, label, name, format, ...others } = props;

	const { closeModal, openModal, registerModal } = useModal();

	const [value, setValue] = useState<Date>();

	useEffect(() => {
		if (props.value) {
			setValue(props.value);
		}
	}, [props.value]);

	const handleCloseModal = (id: ModalId) => closeModal(id);

	const handleBlur = (value: string, e: FocusEvent): void => {
		onBlur?.(new Date(value), e);
	};

	const handleDateClick = (value: Date): void => {
		setValue(value);
		onChange?.(value, null);
	};

	const modalComponent = useCallback(
		(props) => (
			<DatePickerModal
				format={format ?? DefaultDateFormat}
				name={name}
				value={value}
				closeModal={handleCloseModal}
				onChange={handleDateClick}
				{...props}
			/>
		),
		[format, name, value]
	);

	const handleOpenModal = (): void => {
		registerModal({
			id: name,
			title: label,
			style: { width: 400 },
			Component: modalComponent,
		});
		openModal(name);
	};

	return (
		<TextField
			name={name}
			label={label}
			onClick={handleOpenModal}
			onFocus={handleOpenModal}
			onBlur={handleBlur}
			value={value ? formatFn(value, props.format ?? DefaultDateFormat) : undefined}
			{...omit(['value', 'onClick'], others)}
			readonly
		/>
	);
};
