import React, { MouseEvent, useEffect, useMemo, useState } from 'react';
import clsx from 'clsx';

import CircleIcon from '@prokyon/components/Icons/Circle';
import CircleFillIcon from '@prokyon/components/Icons/CircleFill';
import { StatusEnum } from '@prokyon/types/common';
import { InputElementType } from '@prokyon/types/form';
import { Option } from '@prokyon/types/options';

export type OptionViewProps = {
	icons?: { [key: string]: React.ReactNode };
};

export type InnerOptionsProps = {
	onClick: ({ code, name }: Option, e: MouseEvent<HTMLDivElement>) => void;
	option: Option;
	value: string;
};

export type RadioButtonProps<V = Option> = Omit<
	{
		horizontal?: boolean;
		options: Option[];
		/**
		 * Horizontal prop is ignored and flex-direction is set to column for mobile
		 */
		responsive?: boolean;
		/**
		 * Is called per each item in options
		 */
		view?: React.ReactElement<OptionViewProps>;
	} & InputElementType<V>,
	'onFocus' | 'onBlur' | 'onChange' | 'disabled'
>;

export const RadioButton: React.FC<RadioButtonProps> = (props) => {
	const { onClick, horizontal = false, label, options, view, status = StatusEnum.none, responsive, name } = props;

	const [value, setValue] = useState<Option['code'] | null>(null);

	useEffect(() => {
		if (props.value) {
			setValue(props.value.code);
		}
	}, [props.value]);

	const handleClick = (option: Option, e: MouseEvent<HTMLDivElement>): void => {
		onClick?.(option, e);
		setValue(option.code);
	};

	const boxClassName = clsx('text-xl field-radio', {
		'field-radio--info': status === StatusEnum.info,
		'field-radio--warning': status === StatusEnum.warning,
		'field-radio--error': status === StatusEnum.error,
	});

	const elements = useMemo(() => {
		return options.map(({ code, name, disabled }): React.ReactElement => {
			return (
				<div
					key={code}
					className={clsx({
						'mr-3': horizontal,
					})}>
					{view ? (
						React.cloneElement(view, {
							option: { code, name, disabled },
							onClick: handleClick,
							value,
						} as any)
					) : (
						<div
							className={clsx('flex field-radio', { 'field-radio--disabled': disabled })}
							onClick={(e) => (!disabled ? handleClick({ code, name, disabled }, e) : null)}>
							{value === code && !disabled ? (
								<CircleFillIcon className={boxClassName} />
							) : (
								<CircleIcon className={boxClassName} />
							)}
							<div className="ml-1">{name}</div>
						</div>
					)}
				</div>
			);
		});
	}, [value, options, name, boxClassName]);

	return (
		<div className="field-container radiobutton-container">
			{label && <div className="field-label cursor-default mr-2 self-start">{label}</div>}
			<div
				className={clsx(
					'flex',
					responsive
						? 'flex-col sm:flex-row'
						: {
								'flex-col mt-2': !horizontal,
								'flex-row': horizontal,
						  }
				)}>
				{elements}
			</div>
		</div>
	);
};
