import React, { ReactChild, ReactNode, useEffect, useMemo, useState } from 'react';
import { isNil } from 'ramda';
import clsx from 'clsx';
import { useLocation } from 'wouter';

import MenuIcon from '@prokyon/components/Icons/Menu';
import RightToBracketIcon from '@prokyon/components/Icons/RightToBracket';
import Modals from '@prokyon/components/Modal/components/Modals';
import { Toasters } from '@prokyon/components/Toaster';
import { SIZES, useMediaQuery } from '@prokyon/hooks/useMediaQuery';
import { isNilOrEmpty } from '@prokyon/utils';

import { MenuItem } from '../types';
import { Item } from './Item';

export type SkeletonProps = {
	children: ReactChild;
	components?: {
		footer?: React.ReactElement;
	};
	items?: {
		logo?: ReactNode;
		side?: MenuItem[];
		top?: MenuItem[];
		user?: MenuItem[];
	};
};

export const Skeleton: React.FC<SkeletonProps> = (props) => {
	const { children, components = {}, items } = props;
	const { footer } = components;

	const [, setLocation] = useLocation();

	const enabledSide = !isNilOrEmpty(items?.side);
	const enabledTop = !isNilOrEmpty(items?.top);

	const { isMaxMatch: isMobile } = useMediaQuery({ size: SIZES.md });
	const { isMaxMatch: isTablet } = useMediaQuery({ size: SIZES.lg });

	const [sideCollapsed, setSideCollapsed] = useState(isMobile || isTablet);
	const [topCollapsed, setTopCollapsed] = useState(isMobile);

	useEffect(() => {
		setSideCollapsed(isMobile || isTablet);
		setTopCollapsed(isMobile);
	}, [isMobile, isTablet]);

	const toggleSide = () => {
		setSideCollapsed(!sideCollapsed);
	};

	const toggleTop = () => {
		setTopCollapsed(!topCollapsed);
	};

	const openLink = ({ href, external, onClick }: MenuItem) => {
		if (!isNil(onClick)) {
			onClick();
		} else {
			if (external) {
				window.open(href, '_blank');
			} else if (href) {
				setLocation(href);
			}
		}
	};

	const topItems = useMemo(() => {
		return items?.top?.map((item, i) => <Item key={item.href ?? i} onClick={openLink} item={item} />);
	}, [items?.top]);

	const userItems = useMemo(() => {
		return items?.user?.map((item, i) => <Item key={item.href ?? i} onClick={openLink} item={item} />);
	}, [items?.user]);

	const sideItems = useMemo(() => {
		return items?.side?.map((item, i) => (
			<Item key={item.href ?? i} onClick={openLink} item={item} collapsed={sideCollapsed} />
		));
	}, [items?.side, sideCollapsed]);

	return (
		<div className="page">
			{enabledTop && (
				<div className="page-top">
					<div className="page-top-content">
						<div className="page-top-control">
							{items?.logo && <div className="page-top-logo">{items?.logo}</div>}
							{isMobile && <MenuIcon className="page-top-toggler" onClick={toggleTop} />}
						</div>
						{!topCollapsed && (
							<>
								<div className="ml-auto" />
								{topItems}
								<div className="ml-auto" />
								{userItems}
							</>
						)}
					</div>
				</div>
			)}
			<div className={clsx('page-main', { flex: enabledSide })}>
				{enabledSide && (
					<div
						className={clsx('side', {
							'side--collapsed': sideCollapsed,
							'side--expand': !sideCollapsed,
						})}>
						<RightToBracketIcon
							className={clsx('side-toggler', { 'rotate-180': !sideCollapsed })}
							onClick={toggleSide}
						/>
						{sideItems}
					</div>
				)}
				<div className="content">
					{children}
					<div className="mt-4">{footer}</div>
				</div>
			</div>
			<Modals />
			<Toasters />
		</div>
	);
};
